@@ -5,7 +5,7 @@ Author: Matthew Turk <matthewturk@gmail.com>
 Affiliation: KIPAC/SLAC/Stanford
 Homepage: http://yt.enzotools.org/
 License:
-  Copyright (C) 2007-2008 Matthew Turk.  All Rights Reserved.
+  Copyright (C) 2007-2009 Matthew Turk.  All Rights Reserved.
 
   This file is part of yt.
 
@@ -30,6 +30,9 @@ _field_mapping = {
     "hybrid_radius": ("RadiusCode", "ParticleRadiusCode"),
                  }
 
+class EmptyProfileData(Exception):
+    pass
+
 def preserve_source_parameters(func):
     def save_state(*args, **kwargs):
         # Temporarily replace the 'field_parameters' for a
@@ -48,49 +51,77 @@ def preserve_source_parameters(func):
 
 # Note we do not inherit from EnzoData.
 # We could, but I think we instead want to deal with the root datasource.
-class BinnedProfile:
+class BinnedProfile(ParallelAnalysisInterface):
     def __init__(self, data_source, lazy_reader):
         self._data_source = data_source
+        self.pf = data_source.pf
         self._data = {}
+        self._pdata = {}
         self._lazy_reader = lazy_reader
 
+    @property
+    def hierarchy(self):
+        return self.pf.hierarchy
+
+    def _get_dependencies(self, fields):
+        return ParallelAnalysisInterface._get_dependencies(
+                    self, fields + self._get_bin_fields())
+
+    def _initialize_parallel(self, fields):
+        g_objs = [g for g in self._get_grid_objs()]
+        self._preload(g_objs, self._get_dependencies(fields),
+                      self._data_source.hierarchy.queue)
+
     def _lazy_add_fields(self, fields, weight, accumulation):
-        data = {}         # final results will go here
-        weight_data = {}  # we need to track the weights as we go
+        self._ngrids = 0
+        self.__data = {}         # final results will go here
+        self.__weight_data = {}  # we need to track the weights as we go
         for field in fields:
-            data[field] = self._get_empty_field()
-            weight_data[field] = self._get_empty_field()
-        used = self._get_empty_field().astype('bool')
-        pbar = get_pbar('Binning grids', len(self._data_source._grids))
-        for gi,grid in enumerate(self._data_source._grids):
-            pbar.update(gi)
-            args = self._get_bins(grid, check_cut=True)
-            if not args: # No bins returned for this grid, so forget it!
+            self.__data[field] = self._get_empty_field()
+            self.__weight_data[field] = self._get_empty_field()
+        self.__used = self._get_empty_field().astype('bool')
+        #pbar = get_pbar('Binning grids', len(self._data_source._grids))
+        for gi,grid in enumerate(self._get_grids(fields)):
+            self._ngrids += 1
+            #pbar.update(gi)
+            try:
+                args = self._get_bins(grid, check_cut=True)
+            except EmptyProfileData:
+                # No bins returned for this grid, so forget it!
                 continue
             for field in fields:
                 # We get back field values, weight values, used bins
                 f, w, u = self._bin_field(grid, field, weight, accumulation,
                                           args=args, check_cut=True)
-                data[field] += f        # running total
-                weight_data[field] += w # running total
-                used = (used | u)       # running 'or'
+                self.__data[field] += f        # running total
+                self.__weight_data[field] += w # running total
+                self.__used = (self.__used | u)       # running 'or'
             grid.clear_data()
-        pbar.finish()
-        ub = na.where(used)
+        # When the loop completes the parallel finalizer gets called
+        #pbar.finish()
+        ub = na.where(self.__used)
         for field in fields:
             if weight: # Now, at the end, we divide out.
-                data[field][ub] /= weight_data[field][ub]
-            self[field] = data[field]
-        self["UsedBins"] = used
+                self.__data[field][ub] /= self.__weight_data[field][ub]
+            self[field] = self.__data[field]
+        self["UsedBins"] = self.__used
+        del self.__data, self.__weight_data, self.__used
+
+    def _finalize_parallel(self):
+        for key in self.__data:
+            self.__data[key] = self._mpi_allsum(self.__data[key])
+        for key in self.__weight_data:
+            self.__weight_data[key] = self._mpi_allsum(self.__weight_data[key])
+        self.__used = self._mpi_allsum(self.__used)
 
     def _unlazy_add_fields(self, fields, weight, accumulation):
         for field in fields:
             f, w, u = self._bin_field(self._data_source, field, weight,
                                       accumulation, self._args, check_cut = False)
-            ub = na.where(u)
             if weight:
-                f[ub] /= w[ub]
+                f[u] /= w[u]
             self[field] = f
+        self["myweight"] = w
         self["UsedBins"] = u
 
     def add_fields(self, fields, weight = "CellMassMsun", accumulation = False):
@@ -117,27 +148,32 @@ class BinnedProfile:
     def __setitem__(self, key, value):
         self._data[key] = value
 
-    def _get_field(self, source, field, check_cut):
+    def _get_field(self, source, this_field, check_cut):
         # This is where we will iterate to get all contributions to a field
         # which is how we will implement hybrid particle/cell fields
         # but...  we default to just the field.
         data = []
-        for field in _field_mapping.get(field, (field,)):
+        for field in _field_mapping.get(this_field, (this_field,)):
+            pointI = None
             if check_cut:
-                if field in fieldInfo and fieldInfo[field].particle_type:
+                if field in self.pf.field_info \
+                    and self.pf.field_info[field].particle_type:
                     pointI = self._data_source._get_particle_indices(source)
                 else:
                     pointI = self._data_source._get_point_indices(source)
-            else:
-                pointI = slice(None)
             data.append(source[field][pointI].ravel().astype('float64'))
         return na.concatenate(data, axis=0)
 
+    def _fix_pickle(self):
+        if isinstance(self._data_source, tuple):
+            self._data_source = self._data_source[1]
+
 # @todo: Fix accumulation with overriding
 class BinnedProfile1D(BinnedProfile):
     def __init__(self, data_source, n_bins, bin_field,
                  lower_bound, upper_bound,
-                 log_space = True, lazy_reader=False):
+                 log_space = True, lazy_reader=False,
+                 left_collect = False):
         """
         A 'Profile' produces either a weighted (or unweighted) average or a
         straight sum of a field in a bin defined by another field.  In the case
@@ -152,13 +188,18 @@ class BinnedProfile1D(BinnedProfile):
         BinnedProfile.__init__(self, data_source, lazy_reader)
         self.bin_field = bin_field
         self._x_log = log_space
+        self.left_collect = left_collect
         # Get our bins
         if log_space:
             func = na.logspace
             lower_bound, upper_bound = na.log10(lower_bound), na.log10(upper_bound)
         else:
             func = na.linspace
-        self[bin_field] = func(lower_bound, upper_bound, n_bins)
+        # These are the bin *edges*
+        self._bins = func(lower_bound, upper_bound, n_bins + 1)
+
+        # These are the bin *left edges*
+        self[bin_field] = self._bins[:-1]
 
         # If we are not being memory-conservative, grab all the bins
         # and the inverse indices right now.
@@ -200,15 +241,19 @@ class BinnedProfile1D(BinnedProfile):
     def _get_bins(self, source, check_cut=False):
         source_data = self._get_field(source, self.bin_field, check_cut)
         if source_data.size == 0: # Nothing for us here.
-            return
+            raise EmptyProfileData()
         # Truncate at boundaries.
-        mi = na.where( (source_data > self[self.bin_field].min())
-                     & (source_data < self[self.bin_field].max()))
+        if self.left_collect:
+            mi = na.where(source_data < self._bins.max())
+        else:
+            mi = na.where( (source_data > self._bins.min())
+                         & (source_data < self._bins.max()))
         sd = source_data[mi]
         if sd.size == 0:
-            return
+            raise EmptyProfileData()
         # Stick the bins into our fixed bins, set at initialization
-        bin_indices = na.digitize(sd, self[self.bin_field])
+        bin_indices = na.digitize(sd, self._bins) - 1
+        if self.left_collect: bin_indices = na.maximum(0, bin_indices)
         # Now we set up our inverse bin indices
         inv_bin_indices = {}
         for bin in range(self[self.bin_field].size):
@@ -226,11 +271,14 @@ class BinnedProfile1D(BinnedProfile):
             fid.write("\n")
         fid.close()
 
+    def _get_bin_fields(self):
+        return [self.bin_field]
+
 class BinnedProfile2D(BinnedProfile):
     def __init__(self, data_source,
                  x_n_bins, x_bin_field, x_lower_bound, x_upper_bound, x_log,
                  y_n_bins, y_bin_field, y_lower_bound, y_upper_bound, y_log,
-                 lazy_reader=False):
+                 lazy_reader=False, left_collect=False):
         """
         A 'Profile' produces either a weighted (or unweighted) average or a
         straight sum of a field in a bin defined by two other fields.  In the case
@@ -249,16 +297,19 @@ class BinnedProfile2D(BinnedProfile):
         self.y_bin_field = y_bin_field
         self._x_log = x_log
         self._y_log = y_log
-        if x_log: self[x_bin_field] = na.logspace(na.log10(x_lower_bound*0.99),
-                                                  na.log10(x_upper_bound*1.01),
-                                                  x_n_bins)
-        else: self[x_bin_field] = na.linspace(
-                x_lower_bound*0.99, x_upper_bound*1.01, x_n_bins)
-        if y_log: self[y_bin_field] = na.logspace(na.log10(y_lower_bound*0.99),
-                                                  na.log10(y_upper_bound*1.01),
-                                                  y_n_bins)
-        else: self[y_bin_field] = na.linspace(
-                y_lower_bound*0.99, y_upper_bound*1.01, y_n_bins)
+        self.left_collect = left_collect
+
+        func = {True:na.logspace, False:na.linspace}[x_log]
+        bounds = fix_bounds(x_upper_bound, x_lower_bound, x_log)
+        self._x_bins = func(bounds[0], bounds[1], x_n_bins + 1)
+        self[x_bin_field] = self._x_bins[:-1]
+
+        func = {True:na.logspace, False:na.linspace}[y_log]
+        bounds = fix_bounds(y_upper_bound, y_lower_bound, y_log)
+        self._y_bins = func(bounds[0], bounds[1], y_n_bins + 1)
+        self[y_bin_field] = self._y_bins[:-1]
+
+
         if na.any(na.isnan(self[x_bin_field])) \
             or na.any(na.isnan(self[y_bin_field])):
             mylog.error("Your min/max values for x, y have given me a nan.")
@@ -304,17 +355,24 @@ class BinnedProfile2D(BinnedProfile):
         source_data_x = self._get_field(source, self.x_bin_field, check_cut)
         source_data_y = self._get_field(source, self.y_bin_field, check_cut)
         if source_data_x.size == 0:
-            return
-        mi = na.where( (source_data_x > self[self.x_bin_field].min())
-                     & (source_data_x < self[self.x_bin_field].max())
-                     & (source_data_y > self[self.y_bin_field].min())
-                     & (source_data_y < self[self.y_bin_field].max()))
+            raise EmptyProfileData()
+        if self.left_collect:
+            mi = na.where( (source_data_x < self._x_bins.max())
+                         & (source_data_y < self._y_bins.max()))
+        else:
+            mi = na.where( (source_data_x > self._x_bins.min())
+                         & (source_data_x < self._x_bins.max())
+                         & (source_data_y > self._y_bins.min())
+                         & (source_data_y < self._y_bins.max()))
         sd_x = source_data_x[mi]
         sd_y = source_data_y[mi]
         if sd_x.size == 0 or sd_y.size == 0:
-            return
-        bin_indices_x = na.digitize(sd_x, self[self.x_bin_field])
-        bin_indices_y = na.digitize(sd_y, self[self.y_bin_field])
+            raise EmptyProfileData()
+        bin_indices_x = na.digitize(sd_x, self._x_bins) - 1
+        bin_indices_y = na.digitize(sd_y, self._y_bins) - 1
+        if self.left_collect:
+            bin_indices_x = na.maximum(bin_indices_x, 0)
+            bin_indices_y = na.maximum(bin_indices_y, 0)
         # Now we set up our inverse bin indices
         return (mi, bin_indices_x, bin_indices_y)
 
@@ -338,6 +396,13 @@ class BinnedProfile2D(BinnedProfile):
             fid.write("\n")
         fid.close()
 
+    def _get_bin_fields(self):
+        return [self.x_bin_field, self.y_bin_field]
+
+def fix_bounds(upper, lower, logit):
+    if logit: return na.log10(upper), na.log10(lower)
+    return upper, lower
+
 class BinnedProfile3D(BinnedProfile):
     def __init__(self, data_source,
                  x_n_bins, x_bin_field, x_lower_bound, x_upper_bound, x_log,
@@ -351,21 +416,22 @@ class BinnedProfile3D(BinnedProfile):
         self._x_log = x_log
         self._y_log = y_log
         self._z_log = z_log
-        if x_log: self[x_bin_field] = na.logspace(na.log10(x_lower_bound*0.99),
-                                                  na.log10(x_upper_bound*1.01),
-                                                  x_n_bins)
-        else: self[x_bin_field] = na.linspace(
-                x_lower_bound*0.99, x_upper_bound*1.01, x_n_bins)
-        if y_log: self[y_bin_field] = na.logspace(na.log10(y_lower_bound*0.99),
-                                                  na.log10(y_upper_bound*1.01),
-                                                  y_n_bins)
-        else: self[y_bin_field] = na.linspace(
-                y_lower_bound*0.99, y_upper_bound*1.01, y_n_bins)
-        if z_log: self[z_bin_field] = na.logspace(na.log10(z_lower_bound*0.99),
-                                                  na.log10(z_upper_bound*1.01),
-                                                  z_n_bins)
-        else: self[z_bin_field] = na.linspace(
-                z_lower_bound*0.99, z_upper_bound*1.01, z_n_bins)
+
+        func = {True:na.logspace, False:na.linspace}[x_log]
+        bounds = fix_bounds(x_upper_bound, x_lower_bound, x_log)
+        self._x_bins = func(bounds[0], bounds[1], x_n_bins + 1)
+        self[x_bin_field] = self._x_bins[:-1]
+
+        func = {True:na.logspace, False:na.linspace}[y_log]
+        bounds = fix_bounds(y_upper_bound, y_lower_bound, y_log)
+        self._y_bins = func(bounds[0], bounds[1], y_n_bins + 1)
+        self[y_bin_field] = self._y_bins[:-1]
+
+        func = {True:na.logspace, False:na.linspace}[z_log]
+        bounds = fix_bounds(z_upper_bound, z_lower_bound, z_log)
+        self._z_bins = func(bounds[0], bounds[1], z_n_bins + 1)
+        self[z_bin_field] = self._z_bins[:-1]
+
         if na.any(na.isnan(self[x_bin_field])) \
             or na.any(na.isnan(self[y_bin_field])) \
             or na.any(na.isnan(self[z_bin_field])):
@@ -418,27 +484,30 @@ class BinnedProfile3D(BinnedProfile):
         source_data_y = self._get_field(source, self.y_bin_field, check_cut)
         source_data_y = self._get_field(source, self.z_bin_field, check_cut)
         if source_data_x.size == 0:
-            return
-        mi = na.where( (source_data_x > self[self.x_bin_field].min())
-                     & (source_data_x < self[self.x_bin_field].max())
-                     & (source_data_y > self[self.y_bin_field].min())
-                     & (source_data_y < self[self.y_bin_field].max())
-                     & (source_data_z > self[self.z_bin_field].min())
-                     & (source_data_z < self[self.z_bin_field].max()))
+            raise EmptyProfileData()
+        mi = ( (source_data_x > self._x_bins.min())
+             & (source_data_x < self._x_bins.max())
+             & (source_data_y > self._y_bins.min())
+             & (source_data_y < self._y_bins.max())
+             & (source_data_z > self._z_bins.min())
+             & (source_data_z < self._z_bins.max()))
         sd_x = source_data_x[mi]
         sd_y = source_data_y[mi]
         sd_z = source_data_z[mi]
         if sd_x.size == 0 or sd_y.size == 0 or sd_z.size == 0:
-            return
-        bin_indices_x = na.digitize(sd_x, self[self.x_bin_field])
-        bin_indices_y = na.digitize(sd_y, self[self.y_bin_field])
-        bin_indices_z = na.digitize(sd_z, self[self.z_bin_field])
+            raise EmptyProfileData()
+        bin_indices_x = na.digitize(sd_x, self._x_bins)
+        bin_indices_y = na.digitize(sd_y, self._y_bins)
+        bin_indices_z = na.digitize(sd_z, self._z_bins)
         # Now we set up our inverse bin indices
         return (mi, bin_indices_x, bin_indices_y, bin_indices_z)
 
     def write_out(self, filename, format="%0.16e"):
         pass # Will eventually dump HDF5
 
+    def _get_bin_fields(self):
+        return [self.x_bin_field, self.y_bin_field, self.z_bin_field]
+
     def store_profile(self, name, force=False):
         """
         By identifying the profile with a fixed, user-input *name* we can
